# coding: utf-8

"""
    Merlin

    API Guide for accessing Merlin's model management, deployment, and serving functionalities  # noqa: E501

    OpenAPI spec version: 0.14.0
    
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""


import pprint
import re  # noqa: F401

import six

from client.configuration import Configuration


class PredictionJobConfigModelResult(object):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """

    """
    Attributes:
      swagger_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    swagger_types = {
        'type': 'ResultType',
        'item_type': 'ResultType'
    }

    attribute_map = {
        'type': 'type',
        'item_type': 'item_type'
    }

    def __init__(self, type=None, item_type=None, _configuration=None):  # noqa: E501
        """PredictionJobConfigModelResult - a model defined in Swagger"""  # noqa: E501
        if _configuration is None:
            _configuration = Configuration()
        self._configuration = _configuration

        self._type = None
        self._item_type = None
        self.discriminator = None

        if type is not None:
            self.type = type
        if item_type is not None:
            self.item_type = item_type

    @property
    def type(self):
        """Gets the type of this PredictionJobConfigModelResult.  # noqa: E501


        :return: The type of this PredictionJobConfigModelResult.  # noqa: E501
        :rtype: ResultType
        """
        return self._type

    @type.setter
    def type(self, type):
        """Sets the type of this PredictionJobConfigModelResult.


        :param type: The type of this PredictionJobConfigModelResult.  # noqa: E501
        :type: ResultType
        """

        self._type = type

    @property
    def item_type(self):
        """Gets the item_type of this PredictionJobConfigModelResult.  # noqa: E501


        :return: The item_type of this PredictionJobConfigModelResult.  # noqa: E501
        :rtype: ResultType
        """
        return self._item_type

    @item_type.setter
    def item_type(self, item_type):
        """Sets the item_type of this PredictionJobConfigModelResult.


        :param item_type: The item_type of this PredictionJobConfigModelResult.  # noqa: E501
        :type: ResultType
        """

        self._item_type = item_type

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value
        if issubclass(PredictionJobConfigModelResult, dict):
            for key, value in self.items():
                result[key] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, PredictionJobConfigModelResult):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, PredictionJobConfigModelResult):
            return True

        return self.to_dict() != other.to_dict()
